import _classCallCheck from '@babel/runtime/helpers/esm/classCallCheck';
import _createClass from '@babel/runtime/helpers/esm/createClass';
import { inject, singleton, Module } from 'mana-syringe';
import { ContextService, RenderingPluginContribution } from '@antv/g';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

function __metadata(metadataKey, metadataValue) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
}

// use `self` instead of `window`, account for non-window contexts, such as in Web Workers
// @see https://developer.mozilla.org/en-US/docs/Web/API/Window/self
var supportsTouchEvents = ('ontouchstart' in self);
var supportsPointerEvents = !!self.PointerEvent;

var _class, _temp;

var DOMInteractionPlugin_1;
/**
 * listen to mouse/touch/pointer events on DOM wrapper, trigger pointer events
 */

var DOMInteractionPlugin = DOMInteractionPlugin_1 = (_temp = _class = /*#__PURE__*/function () {
  function DOMInteractionPlugin() {
    _classCallCheck(this, DOMInteractionPlugin);

    this.contextService = void 0;
  }

  _createClass(DOMInteractionPlugin, [{
    key: "apply",
    value: function apply(renderingService) {
      var _this = this;

      var onPointerMove = function onPointerMove(ev) {
        renderingService.hooks.pointerMove.call(ev);
      };

      var onPointerUp = function onPointerUp(ev) {
        renderingService.hooks.pointerUp.call(ev);
      };

      var onPointerDown = function onPointerDown(ev) {
        renderingService.hooks.pointerDown.call(ev);
      };

      var onPointerOver = function onPointerOver(ev) {
        renderingService.hooks.pointerOver.call(ev);
      };

      var onPointerOut = function onPointerOut(ev) {
        renderingService.hooks.pointerOut.call(ev);
      };

      var onPointerWheel = function onPointerWheel(ev) {
        renderingService.hooks.pointerWheel.call(ev);
      };

      renderingService.hooks.init.tap(DOMInteractionPlugin_1.tag, function () {
        var $el = _this.contextService.getDomElement();

        if (supportsPointerEvents) {
          self.document.addEventListener('pointermove', onPointerMove, true);
          $el.addEventListener('pointerdown', onPointerDown, true);
          $el.addEventListener('pointerleave', onPointerOut, true);
          $el.addEventListener('pointerover', onPointerOver, true);
          self.addEventListener('pointerup', onPointerUp, true);
        } else {
          self.document.addEventListener('mousemove', onPointerMove, true);
          $el.addEventListener('mousedown', onPointerDown, true);
          $el.addEventListener('mouseout', onPointerOut, true);
          $el.addEventListener('mouseover', onPointerOver, true);
          self.addEventListener('mouseup', onPointerUp, true);
        } // always look directly for touch events so that we can provide original data
        // In a future version we should change this to being just a fallback and rely solely on
        // PointerEvents whenever available


        if (supportsTouchEvents) {
          $el.addEventListener('touchstart', onPointerDown, true);
          $el.addEventListener('touchend', onPointerUp, true);
          $el.addEventListener('touchmove', onPointerMove, true);
        } // use passive event listeners
        // @see https://zhuanlan.zhihu.com/p/24555031


        $el.addEventListener('wheel', onPointerWheel, {
          passive: true,
          capture: true
        });
      });
      renderingService.hooks.destroy.tap(DOMInteractionPlugin_1.tag, function () {
        var $el = _this.contextService.getDomElement();

        if (supportsPointerEvents) {
          self.document.removeEventListener('pointermove', onPointerMove, true);
          $el.removeEventListener('pointerdown', onPointerDown, true);
          $el.removeEventListener('pointerleave', onPointerOut, true);
          $el.removeEventListener('pointerover', onPointerOver, true);
          self.removeEventListener('pointerup', onPointerUp, true);
        } else {
          self.document.removeEventListener('mousemove', onPointerMove, true);
          $el.removeEventListener('mousedown', onPointerDown, true);
          $el.removeEventListener('mouseout', onPointerOut, true);
          $el.removeEventListener('mouseover', onPointerOver, true);
          self.removeEventListener('mouseup', onPointerUp, true);
        }

        if (supportsTouchEvents) {
          $el.removeEventListener('touchstart', onPointerDown, true);
          $el.removeEventListener('touchend', onPointerUp, true);
          $el.removeEventListener('touchmove', onPointerMove, true);
        }

        $el.removeEventListener('wheel', onPointerWheel, true);
      });
    }
  }]);

  return DOMInteractionPlugin;
}(), _class.tag = 'DOMInteractionPlugin', _temp);

__decorate([inject(ContextService), __metadata("design:type", Object)], DOMInteractionPlugin.prototype, "contextService", void 0);

DOMInteractionPlugin = DOMInteractionPlugin_1 = __decorate([singleton({
  contrib: RenderingPluginContribution
})], DOMInteractionPlugin);

var containerModule = Module(function (register) {
  register(DOMInteractionPlugin);
});
var Plugin = /*#__PURE__*/function () {
  function Plugin() {
    _classCallCheck(this, Plugin);
  }

  _createClass(Plugin, [{
    key: "init",
    value: function init(container) {
      container.load(containerModule);
    }
  }, {
    key: "destroy",
    value: function destroy(container) {// @ts-ignore
      // container.container.unload(containerModule);
      // // container.unload(containerModule);
      // container.remove(DOMInteractionPlugin);
    }
  }]);

  return Plugin;
}();

export { Plugin };
